@@ -16,18 +16,15 @@
 
 __author__ = 'f1ashhimself@gmail.com'
 
-import re
 import ctypes
 import ctypes.wintypes
 import comtypes
 import comtypes.automation
 import comtypes.client
-from inspect import ismethod
-from types import FunctionType
-import xml.dom.minidom
 
 from .mouse import WinMouse
 from ..interfaces.i_element import IElement
+from ..utils.win_utils import WinUtils
 from .. import TooSaltyUISoupException
 
 
@@ -108,7 +105,7 @@ class WinElement(IElement):
 
     _mouse = WinMouse()
 
-    class StateFlag(object):
+    class _StateFlag(object):
         SYSTEM_NORMAL = 0
         SYSTEM_UNAVAILABLE = 0x1
         SYSTEM_SELECTED = 0x2
@@ -143,7 +140,7 @@ class WinElement(IElement):
         SYSTEM_HASPOPUP = 0x40000000
         SYSTEM_VALID = 0x7fffffff
 
-    class SelectionFlag(object):
+    class _SelectionFlag(object):
         NONE = 0
         TAKEFOCUS = 0x1
         TAKESELECTION = 0x2
@@ -170,6 +167,14 @@ class WinElement(IElement):
             return True
 
     def __init__(self, obj_handle, i_object_id):
+        """
+        Constructor.
+
+        Arguments:
+            - obj_handle: instance of i_accessible or window handle.
+            - i_object_id: int, object id.
+        """
+
         if isinstance(obj_handle, comtypes.gen.Accessibility.IAccessible):
             i_accessible = obj_handle
         else:
@@ -182,8 +187,83 @@ class WinElement(IElement):
                 ctypes.byref(i_accessible))
 
         self._i_accessible = i_accessible
-        self.i_object_id = i_object_id
-        self._cache = set()
+        self._i_object_id = i_object_id
+        self._cached_children = set()
+
+    def _check_state(self, state):
+        """
+        Checks state.
+
+        Arguments:
+            - state: int, state flag.
+
+        Returns:
+            - Bool flag indicator.
+        """
+
+        return bool(self._acc_state & state)
+
+    def _find_windows_by_same_proc(self):
+        """
+        Find window by same process id.
+
+        Arguments:
+            - None
+
+        Returns:
+            - list of windows.
+        """
+
+        enum_windows_proc = \
+            ctypes.WINFUNCTYPE(ctypes.c_bool, ctypes.c_long,
+                               ctypes.c_long)
+        self._EnumWindowsCallback.same_proc_handles = set()
+        ctypes.windll.user32.EnumWindows(
+            enum_windows_proc(
+                self._EnumWindowsCallback.callback), self.proc_id)
+
+        if self._hwnd in self._EnumWindowsCallback.same_proc_handles:
+            self._EnumWindowsCallback.same_proc_handles.remove(self._hwnd)
+
+        result = [WinElement(hwnd, 0) for hwnd in
+                  self._EnumWindowsCallback.same_proc_handles]
+
+        return result
+
+    @property
+    def _hwnd(self):
+        """
+        Property for window handler.
+        """
+
+        hwnd = ctypes.c_int()
+        ctypes.oledll.oleacc.WindowFromAccessibleObject(self._i_accessible,
+                                                        ctypes.byref(hwnd))
+
+        return hwnd.value
+
+    @property
+    def _role(self):
+        """
+        Property for element role.
+        """
+
+        obj_child_id = comtypes.automation.VARIANT()
+        obj_child_id.vt = comtypes.automation.VT_I4
+        obj_child_id.value = self._i_object_id
+        obj_role = comtypes.automation.VARIANT()
+        obj_role.vt = comtypes.automation.VT_BSTR
+
+        self._i_accessible._IAccessible__com__get_accRole(obj_child_id,
+                                                          obj_role)
+
+        return obj_role.value
+
+    def _select(self, i_selection):
+        if self._i_object_id:
+            return self._i_accessible.accSelect(i_selection, self._i_object_id)
+        else:
+            return self._i_accessible.accSelect(i_selection)
 
     def click(self, x_offset=0, y_offset=0):
         x, y, w, h = self.acc_location
@@ -213,47 +293,9 @@ class WinElement(IElement):
 
         self._mouse.drag(el_x, el_y, x, y, smooth)
 
-    def check_state(self, state):
-        return self.acc_state & state
-
-    def _find_windows_by_same_proc(self):
-        """
-        Find window by same process id.
-
-        Arguments:
-            - None
-
-        Returns:
-            - list of windows.
-        """
-
-        enum_windows_proc = \
-            ctypes.WINFUNCTYPE(ctypes.c_bool, ctypes.c_long,
-                               ctypes.c_long)
-        self._EnumWindowsCallback.same_proc_handles = set()
-        ctypes.windll.user32.EnumWindows(
-            enum_windows_proc(
-                self._EnumWindowsCallback.callback), self.proc_id)
-
-        if self.hwnd in self._EnumWindowsCallback.same_proc_handles:
-            self._EnumWindowsCallback.same_proc_handles.remove(self.hwnd)
-
-        result = [WinElement(hwnd, 0) for hwnd in
-                  self._EnumWindowsCallback.same_proc_handles]
-
-        return result
-
-    @property
-    def hwnd(self):
-        hwnd = ctypes.c_int()
-        ctypes.oledll.oleacc.WindowFromAccessibleObject(self._i_accessible,
-                                                        ctypes.byref(hwnd))
-
-        return hwnd.value
-
     @property
     def proc_id(self):
-        hwnd = ctypes.c_long(self.hwnd)
+        hwnd = ctypes.c_long(self._hwnd)
         proc_id = ctypes.c_ulong()
         ctypes.windll.user32.GetWindowThreadProcessId(hwnd,
                                                       ctypes.byref(proc_id))
@@ -267,23 +309,19 @@ class WinElement(IElement):
 
     @property
     def is_selected(self):
-        return self.check_state(self.StateFlag.SYSTEM_SELECTED)
-
-    @property
-    def is_pressed(self):
-        return self.check_state(self.StateFlag.SYSTEM_PRESSED)
+        return self._check_state(self._StateFlag.SYSTEM_SELECTED)
 
     @property
     def is_checked(self):
-        return self.check_state(self.StateFlag.SYSTEM_CHECKED)
+        return self._check_state(self._StateFlag.SYSTEM_CHECKED)
 
     @property
     def is_visible(self):
-        return not self.check_state(self.StateFlag.SYSTEM_INVISIBLE)
+        return not self._check_state(self._StateFlag.SYSTEM_INVISIBLE)
 
     @property
     def is_enabled(self):
-        return not self.check_state(self.StateFlag.SYSTEM_UNAVAILABLE)
+        return not self._check_state(self._StateFlag.SYSTEM_UNAVAILABLE)
 
     @property
     def acc_parent_count(self):
@@ -297,46 +335,27 @@ class WinElement(IElement):
 
     @property
     def acc_child_count(self):
-        if self.i_object_id == 0:
-            return self._i_accessible.accChildCount
+        if self._i_object_id == 0:
+            return self._get_child_count_safely(self._i_accessible)
         else:
             return 0
 
-    @property
-    def acc_role(self):
-        obj_child_id = comtypes.automation.VARIANT()
-        obj_child_id.vt = comtypes.automation.VT_I4
-        obj_child_id.value = self.i_object_id
-        obj_role = comtypes.automation.VARIANT()
-        obj_role.vt = comtypes.automation.VT_BSTR
-
-        self._i_accessible._IAccessible__com__get_accRole(obj_child_id,
-                                                          obj_role)
-
-        return obj_role.value
-
     @property
     def acc_name(self):
         obj_child_id = comtypes.automation.VARIANT()
         obj_child_id.vt = comtypes.automation.VT_I4
-        obj_child_id.value = self.i_object_id
+        obj_child_id.value = self._i_object_id
 
         obj_name = comtypes.automation.BSTR()
 
         self._i_accessible._IAccessible__com__get_accName(
             obj_child_id, ctypes.byref(obj_name))
+        result = obj_name.value or ''
 
-        return obj_name.value
-
-    def set_name(self, name):
-        obj_child_id = comtypes.automation.VARIANT()
-        obj_child_id.vt = comtypes.automation.VT_I4
-        obj_child_id.value = self.i_object_id
-
-        self._i_accessible._IAccessible__com__set_accName(obj_child_id, name)
+        return WinUtils.replace_inappropriate_symbols(result)
 
     def set_focus(self):
-        self.acc_select(self.SelectionFlag.TAKEFOCUS)
+        self._select(self._SelectionFlag.TAKEFOCUS)
 
     @property
     def acc_c_name(self):
@@ -346,7 +365,7 @@ class WinElement(IElement):
     def acc_location(self):
         obj_child_id = comtypes.automation.VARIANT()
         obj_child_id.vt = comtypes.automation.VT_I4
-        obj_child_id.value = self.i_object_id
+        obj_child_id.value = self._i_object_id
 
         obj_l, obj_t, obj_w, obj_h = ctypes.c_long(), ctypes.c_long(), \
             ctypes.c_long(), ctypes.c_long()
@@ -363,7 +382,7 @@ class WinElement(IElement):
     def acc_value(self):
         obj_child_id = comtypes.automation.VARIANT()
         obj_child_id.vt = comtypes.automation.VT_I4
-        obj_child_id.value = self.i_object_id
+        obj_child_id.value = self._i_object_id
         obj_bstr_value = comtypes.automation.BSTR()
         self._i_accessible._IAccessible__com__get_accValue(
             obj_child_id, ctypes.byref(obj_bstr_value))
@@ -373,63 +392,27 @@ class WinElement(IElement):
     def set_value(self, value):
         obj_child_id = comtypes.automation.VARIANT()
         obj_child_id.vt = comtypes.automation.VT_I4
-        obj_child_id.value = self.i_object_id
+        obj_child_id.value = self._i_object_id
 
         self._i_accessible._IAccessible__com__set_accValue(obj_child_id, value)
 
-    @property
-    def acc_default_action(self):
-        obj_child_id = comtypes.automation.VARIANT()
-        obj_child_id.vt = comtypes.automation.VT_I4
-        obj_child_id.value = self.i_object_id
-        obj_default_action = comtypes.automation.BSTR()
-        self._i_accessible._IAccessible__com__get_accDefaultAction(
-            obj_child_id, ctypes.byref(obj_default_action))
-
-        return obj_default_action.value
-
     @property
     def acc_description(self):
         obj_child_id = comtypes.automation.VARIANT()
         obj_child_id.vt = comtypes.automation.VT_I4
-        obj_child_id.value = self.i_object_id
+        obj_child_id.value = self._i_object_id
         obj_description = comtypes.automation.BSTR()
         self._i_accessible._IAccessible__com__get_accDescription(
             obj_child_id, ctypes.byref(obj_description))
 
         return obj_description.value
 
-    @property
-    def acc_help(self):
-        obj_child_id = comtypes.automation.VARIANT()
-        obj_child_id.vt = comtypes.automation.VT_I4
-        obj_child_id.value = self.i_object_id
-        obj_help = comtypes.automation.BSTR()
-        self._i_accessible._IAccessible__com__get_accHelp(
-            obj_child_id, ctypes.byref(obj_help))
-
-        return obj_help.value
-
-    @property
-    def acc_help_topic(self):
-        return self._i_accessible.acc_help_topic()
-
-    @property
-    def acc_keyboard_shortcut(self):
-        obj_child_id = comtypes.automation.VARIANT()
-        obj_child_id.vt = comtypes.automation.VT_I4
-        obj_child_id.value = self.i_object_id
-        obj_keyboard_shortcut = comtypes.automation.BSTR()
-        self._i_accessible._IAccessible__com__get_acccKeyboardShortcut(
-            obj_child_id, ctypes.byref(obj_keyboard_shortcut))
-
-        return obj_keyboard_shortcut.value
-
     @property
     def acc_parent(self):
         result = None
         if self._i_accessible.accParent:
-            result = WinElement(self._i_accessible.accParent, self.i_object_id)
+            result = \
+                WinElement(self._i_accessible.accParent, self._i_object_id)
 
         return result
 
@@ -442,43 +425,30 @@ class WinElement(IElement):
         return obj_children.value
 
     @property
-    def acc_state(self):
+    def _acc_state(self):
         obj_child_id = comtypes.automation.VARIANT()
         obj_child_id.vt = comtypes.automation.VT_I4
-        obj_child_id.value = self.i_object_id
+        obj_child_id.value = self._i_object_id
         obj_state = comtypes.automation.VARIANT()
         self._i_accessible._IAccessible__com__get_accState(
             obj_child_id, ctypes.byref(obj_state))
 
         return obj_state.value
 
-    def acc_do_default_action(self):
-        obj_child_id = comtypes.automation.VARIANT()
-        obj_child_id.vt = comtypes.automation.VT_I4
-        obj_child_id.value = self.i_object_id
-
-        self._i_accessible._IAccessible__com_accDoDefaultAction(obj_child_id)
-
     @property
-    def acc_focus(self):
+    def acc_focused_element(self):
         result = None
         if self._i_accessible.accFocus:
-            result = WinElement(self._i_accessible.accFocus, self.i_object_id)
+            result = WinElement(self._i_accessible.accFocus, self._i_object_id)
 
         return result
 
-    def acc_select(self, i_selection):
-        if self.i_object_id:
-            return self._i_accessible.accSelect(i_selection, self.i_object_id)
-        else:
-            return self._i_accessible.accSelect(i_selection)
-
     @property
     def acc_role_name(self):
-        return self._acc_role_name_map.get(self.acc_role, 'unknown')
+        return self._acc_role_name_map.get(self._role, 'unknown')
 
     def __iter__(self):
-        if self.i_object_id > 0:
+        if self._i_object_id > 0:
             raise StopIteration()
 
         obj_acc_child_array = (comtypes.automation.VARIANT *
@@ -500,82 +470,6 @@ class WinElement(IElement):
             else:
                 yield WinElement(self._i_accessible, obj_acc_child.value)
 
-    def __str__(self):
-        result = '[Role: %s(0x%X) | Name: %r | Child count: %d]' % \
-                 (self._acc_role_name_map.get(self.acc_role, 'Unknown'),
-                  self.acc_role,
-                  self.acc_name,
-                  self._i_accessible.accChildCount)
-
-        return result
-
-    @classmethod
-    def _convert_wildcard_to_regex(cls, wildcard):
-        """
-        Converts wildcard to regex.
-
-        Arguments:
-            - wildcard: string, wildcard.
-
-        Returns:
-            - String with regex pattern.
-        """
-
-        regex = re.escape(wildcard)
-        regex = regex.replace(r'\?', r'[\s\S]{1}')
-        regex = regex.replace(r'\*', r'[\s\S]*')
-
-        return '^%s$' % regex
-
-    def _match(self, only_visible, **kwargs):
-        """
-        Match method.
-
-        Arguments:
-            - only_visible: bool, flag that indicates will we search only
-            through visible elements.
-            - role: string or lambda e.g. lambda x: x == 13
-            - name: string or lambda.
-            - c_name: string or lambda.
-            - location: string or lambda.
-            - value: string or lambda.
-            - default_action: string or lambda.
-            - description: string or lambda.
-            - help: string or lambda.
-            - help_topic: string or lambda.
-            - keyboard_shortcut: string or lambda.
-            - parent: string or lambda.
-            - selection: string or lambda.
-            - state: string or lambda.
-            - focus: string or lambda.
-            - role_name: string or lambda.
-
-        Returns:
-            - True if element was matched otherwise False.
-        """
-
-        try:
-            if only_visible and not self.is_visible:
-                return False
-
-            for str_property in kwargs:
-                attr = getattr(self, 'acc_' + str_property)
-                if ismethod(attr):
-                    attr = attr()
-
-                expected_result = kwargs[str_property]
-                if type(expected_result) is FunctionType:
-                    if not expected_result(attr):
-                        return False
-                else:
-                    regex = self._convert_wildcard_to_regex(expected_result)
-                    if not re.match(regex, attr):
-                        return False
-        except:
-            return False
-        else:
-            return True
-
     def __findcacheiter(self, only_visible, **kwargs):
         """
         Find child element in the cache.
@@ -587,7 +481,7 @@ class WinElement(IElement):
             - Yield found element.
         """
 
-        for obj_element in self._cache:
+        for obj_element in self._cached_children:
             if obj_element._match(only_visible, **kwargs):
                 yield obj_element
 
@@ -609,15 +503,14 @@ class WinElement(IElement):
 
         while lst_queue:
             obj_element = lst_queue.pop(0)
-            self._cache.add(obj_element)
+            self._cached_children.add(obj_element)
 
             if obj_element._match(only_visible, **kwargs):
                 yield obj_element
 
-            if self._get_child_count_safely(obj_element._i_accessible):
+            if obj_element.acc_child_count:
                 childs = [el for el in list(obj_element) if
                           el._i_accessible != obj_element._i_accessible]
-
                 lst_queue[:0] = childs
 
     def find(self, only_visible=True, **kwargs):
@@ -648,34 +541,6 @@ class WinElement(IElement):
         except TooSaltyUISoupException:
             return False
 
-    def toxml(self):
-        obj_document = xml.dom.minidom.Document()
-        lst_queue = [(self, obj_document)]
-
-        while lst_queue:
-            obj_element, obj_tree = lst_queue.pop(0)
-            role_name = obj_element.acc_role_name
-            obj_name = obj_element.acc_name
-            str_name = unicode(obj_name) if obj_name else ''
-            str_location = ','.join(str(x) for x in obj_element.acc_location)
-            obj_sub_tree = xml.dom.minidom.Element(role_name)
-            obj_sub_tree.ownerDocument = obj_document
-
-            try:
-                obj_sub_tree.attributes['Name'] = str_name
-            except:
-                obj_sub_tree.attributes['Name'] = \
-                    str_name.encode('unicode-escape')
-
-            obj_sub_tree.attributes['Location'] = str_location
-            obj_tree.appendChild(obj_sub_tree)
-
-            if self._get_child_count_safely(obj_element._i_accessible):
-                for obj_element_child in obj_element:
-                    lst_queue.append((obj_element_child, obj_sub_tree))
-
-        return obj_document.toprettyxml()
-
     def _get_child_count_safely(self, i_accessible):
         """
         Safely gets child count.
